@@ -37,13 +37,14 @@
 #include <libtracker-miners-common/tracker-common.h>
 #include <libtracker-sparql/tracker-ontologies.h>
 #include <libtracker-extract/tracker-extract.h>
-#include <libtracker-miner/tracker-miner.h>
 
 #include "tracker-power.h"
 #include "tracker-miner-files.h"
+#include "tracker-miner-files-methods.h"
 #include "tracker-config.h"
 #include "tracker-storage.h"
 #include "tracker-extract-watchdog.h"
+#include "tracker-utils.h"
 
 #define DISK_SPACE_CHECK_FREQUENCY 10
 #define SECONDS_PER_DAY 86400
@@ -61,6 +62,7 @@
 	G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME "," \
 	G_FILE_ATTRIBUTE_STANDARD_SIZE "," \
 	G_FILE_ATTRIBUTE_TIME_MODIFIED "," \
+	G_FILE_ATTRIBUTE_TIME_CREATED "," \
 	G_FILE_ATTRIBUTE_TIME_ACCESS
 
 #define TRACKER_MINER_FILES_GET_PRIVATE(o) (tracker_miner_files_get_instance_private (TRACKER_MINER_FILES (o)))
@@ -797,15 +799,14 @@ set_up_mount_point_cb (GObject      *source,
                        GAsyncResult *result,
                        gpointer      user_data)
 {
-	TrackerSparqlConnection *connection = TRACKER_SPARQL_CONNECTION (source);
-	GError *error = NULL;
+	g_autofree GError *error = NULL;
 
-	tracker_sparql_connection_update_finish (connection, result, &error);
+	tracker_sparql_statement_update_finish (TRACKER_SPARQL_STATEMENT (source),
+	                                        result, &error);
 
 	if (error) {
 		g_critical ("Could not set mount point in database, %s",
 		            error->message);
-		g_error_free (error);
 	}
 }
 
@@ -813,61 +814,37 @@ static void
 set_up_mount_point (TrackerMinerFiles *miner,
                     GFile             *mount_point,
                     gboolean           mounted,
-                    GString           *accumulator)
+                    TrackerBatch      *batch)
 {
-	GString *queries;
-	gchar *uri;
+	TrackerSparqlConnection *conn;
+	g_autoptr (TrackerSparqlStatement) stmt = NULL;
+	g_autofree gchar *now = NULL, *uri = NULL;
 
-	queries = g_string_new ("WITH " DEFAULT_GRAPH " ");
 	uri = g_file_get_uri (mount_point);
+	now = tracker_date_to_string (time (NULL));
 
-	if (mounted) {
-		g_debug ("Mount point state (MOUNTED) being set in DB for mount_point '%s'",
-		         uri);
+	g_debug ("Mount point state (%s) being set in DB for mount_point '%s'",
+	         mounted ? "MOUNTED" : "UNMOUNTED",
+	         uri);
 
-		g_string_append (queries,
-				 "DELETE { ?u tracker:unmountDate ?date ;"
-				 "            tracker:available ?avail } "
-				 "INSERT { ?u tracker:available true } ");
+	conn = tracker_miner_get_connection (TRACKER_MINER (miner));
+	stmt = tracker_load_statement (conn, "update-mountpoint.rq", NULL);
+
+	if (batch) {
+		tracker_batch_add_statement (batch, stmt,
+		                             "mountPoint", G_TYPE_STRING, uri,
+		                             "mounted", G_TYPE_BOOLEAN, mounted,
+		                             "currentDate", G_TYPE_STRING, now,
+		                             NULL);
 	} else {
-		gchar *now;
-
-		g_debug ("Mount point state (UNMOUNTED) being set in DB for URI '%s'",
-		         uri);
-
-		now = tracker_date_to_string (time (NULL));
-
-		g_string_append_printf (queries,
-		                        "DELETE { ?u tracker:unmountDate ?date ;"
-		                        "            tracker:available ?avail } "
-		                        "INSERT { ?u tracker:unmountDate \"%s\" ; "
-					"            tracker:available false } ",
-		                        now);
-
-		g_free (now);
+		tracker_sparql_statement_bind_string (stmt, "mountPoint", uri);
+		tracker_sparql_statement_bind_boolean (stmt, "mounted", mounted);
+		tracker_sparql_statement_bind_string (stmt, "currentDate", now);
+		tracker_sparql_statement_update_async (stmt,
+		                                       NULL,
+		                                       set_up_mount_point_cb,
+		                                       NULL);
 	}
-
-	g_string_append_printf (queries,
-				"WHERE { <%s> a nfo:FileDataObject ; "
-				"             nie:interpretedAs/"
-				"             nie:rootElementOf ?u . "
-				"        ?u tracker:available ?avail . "
-				"        OPTIONAL { ?u tracker:unmountDate ?date } "
-				"}",
-				uri);
-	g_free (uri);
-
-	if (accumulator) {
-		g_string_append_printf (accumulator, "%s ", queries->str);
-	} else {
-		tracker_sparql_connection_update_async (tracker_miner_get_connection (TRACKER_MINER (miner)),
-		                                        queries->str,
-		                                        NULL,
-		                                        set_up_mount_point_cb,
-		                                        NULL);
-	}
-
-	g_string_free (queries, TRUE);
 }
 
 static void
@@ -875,16 +852,13 @@ init_mount_points_cb (GObject      *source,
                       GAsyncResult *result,
                       gpointer      user_data)
 {
-	GError *error = NULL;
+	g_autofree GError *error = NULL;
 
-	tracker_sparql_connection_update_finish (TRACKER_SPARQL_CONNECTION (source),
-	                                         result,
-	                                         &error);
+	tracker_batch_execute_finish (TRACKER_BATCH (source), result, &error);
 
 	if (error) {
 		g_critical ("Could not initialize currently active mount points: %s",
 		            error->message);
-		g_error_free (error);
 	} else {
 		/* Mount points correctly initialized */
 		(TRACKER_MINER_FILES (user_data))->private->mount_points_initialized = TRUE;
@@ -900,28 +874,28 @@ static void
 init_mount_points (TrackerMinerFiles *miner_files)
 {
 	TrackerMiner *miner = TRACKER_MINER (miner_files);
+	TrackerSparqlConnection *conn;
+	TrackerSparqlStatement *stmt;
 	TrackerMinerFilesPrivate *priv;
 	GHashTable *volumes;
 	GHashTableIter iter;
 	gpointer key, value;
-	GString *accumulator;
+	g_autoptr (TrackerBatch) batch = NULL;
 	GError *error = NULL;
-	TrackerSparqlCursor *cursor;
+	TrackerSparqlCursor *cursor = NULL;
 	GSList *mounts, *l;
 	GFile *file;
 
 	g_debug ("Initializing mount points...");
 
-	/* First, get all mounted volumes, according to tracker-store (SYNC!) */
-	cursor = tracker_sparql_connection_query (tracker_miner_get_connection (miner),
-	                                          "SELECT ?f WHERE { "
-	                                          "  ?v a tracker:IndexedFolder ; "
-	                                          "     tracker:isRemovable true; "
-	                                          "     tracker:available true . "
-	                                          "  ?f a nfo:FileDataObject ; "
-	                                          "     nie:interpretedAs/nie:rootElementOf ?v . "
-	                                          "}",
-	                                          NULL, &error);
+	conn = tracker_miner_get_connection (miner);
+	stmt = tracker_load_statement (conn, "get-index-roots.rq", &error);
+
+	if (stmt) {
+		/* First, get all mounted volumes, according to tracker-store (SYNC!) */
+		cursor = tracker_sparql_statement_execute (stmt, NULL, &error);
+	}
+
 	if (error) {
 		g_critical ("Could not obtain the mounted volumes: %s", error->message);
 		g_error_free (error);
@@ -989,7 +963,7 @@ init_mount_points (TrackerMinerFiles *miner_files)
 		g_slist_free (mounts);
 	}
 
-	accumulator = g_string_new (NULL);
+	batch = tracker_sparql_connection_create_batch (conn);
 	g_hash_table_iter_init (&iter, volumes);
 
 	/* Finally, set up volumes based on the composed info */
@@ -1008,7 +982,7 @@ init_mount_points (TrackerMinerFiles *miner_files)
 			set_up_mount_point (TRACKER_MINER_FILES (miner),
 			                    file,
 			                    TRUE,
-			                    accumulator);
+			                    batch);
 
 			if (mount_point) {
 				TrackerIndexingTree *indexing_tree;
@@ -1038,7 +1012,7 @@ init_mount_points (TrackerMinerFiles *miner_files)
 			set_up_mount_point (TRACKER_MINER_FILES (miner),
 			                    file,
 			                    FALSE,
-			                    accumulator);
+			                    batch);
 			/* There's no need to force mtime check in these inconsistent
 			 * mount points, as they are not mounted right now. */
 		}
@@ -1046,19 +1020,11 @@ init_mount_points (TrackerMinerFiles *miner_files)
 		g_free (mount_point);
 	}
 
-	if (accumulator->str[0] != '\0') {
-		tracker_sparql_connection_update_async (tracker_miner_get_connection (miner),
-		                                        accumulator->str,
-		                                        NULL,
-		                                        init_mount_points_cb,
-		                                        miner);
-	} else {
-		/* Note. Not initializing stale volume removal timeout because
-		 * we do not have the configuration setup yet */
-		(TRACKER_MINER_FILES (miner))->private->mount_points_initialized = TRUE;
-	}
+	tracker_batch_execute_async (batch,
+	                             NULL,
+	                             init_mount_points_cb,
+	                             miner);
 
-	g_string_free (accumulator, TRUE);
 	g_hash_table_unref (volumes);
 }
 
@@ -1274,7 +1240,7 @@ check_battery_status (TrackerMinerFiles *mf)
 	on_battery = tracker_power_get_on_battery (mf->private->power);
 
 	if (!on_battery) {
-		g_message ("Running on AC power");
+		g_debug ("Running on AC power");
 		should_pause = FALSE;
 		should_throttle = FALSE;
 	} else if (on_low_battery) {
@@ -1290,15 +1256,15 @@ check_battery_status (TrackerMinerFiles *mf)
 				g_message ("Running on battery, but not enabled, pausing");
 				should_pause = TRUE;
 			} else if (tracker_miner_files_get_first_index_done (mf)) {
-				g_message ("Running on battery and first-time index "
-				           "already done, pausing");
+				g_debug ("Running on battery and first-time index "
+				         "already done, pausing");
 				should_pause = TRUE;
 			} else {
-				g_message ("Running on battery, but first-time index not "
-				           "already finished, keeping on");
+				g_debug ("Running on battery, but first-time index not "
+				         "already finished, keeping on");
 			}
 		} else {
-			g_message ("Running on battery");
+			g_debug ("Running on battery");
 		}
 	}
 
@@ -1387,11 +1353,11 @@ mount_pre_unmount_cb (GVolumeMonitor    *volume_monitor,
 
 	indexing_tree = tracker_miner_fs_get_indexing_tree (TRACKER_MINER_FS (mf));
 	tracker_indexing_tree_remove (indexing_tree, mount_root);
-	g_object_unref (mount_root);
 
 	/* Set mount point status in tracker-store */
 	set_up_mount_point (mf, mount_root, FALSE, NULL);
 
+	g_object_unref (mount_root);
 	g_free (uri);
 }
 
@@ -1960,260 +1926,25 @@ index_applications_changed_cb (GObject    *gobject,
 }
 
 static void
-miner_files_add_to_datasource (TrackerMinerFiles *mf,
-                               GFile             *file,
-                               TrackerResource   *resource,
-                               TrackerResource   *element_resource)
+miner_files_process_file (TrackerMinerFS       *fs,
+                          GFile                *file,
+                          GFileInfo            *info,
+                          TrackerSparqlBuffer  *buffer,
+                          gboolean              create)
 {
-	TrackerIndexingTree *indexing_tree;
-	TrackerMinerFS *fs = TRACKER_MINER_FS (mf);
-
-	indexing_tree = tracker_miner_fs_get_indexing_tree (fs);
-
-	if (tracker_indexing_tree_file_is_root (indexing_tree, file)) {
-		tracker_resource_set_relation (resource, "nie:dataSource", element_resource);
-	} else {
-		gchar *identifier = NULL;
-		GFile *root;
-
-		root = tracker_indexing_tree_get_root (indexing_tree, file, NULL);
-
-		if (root)
-			identifier = tracker_miner_fs_get_identifier (fs, root, FALSE, TRUE, NULL);
-
-		if (identifier)
-			tracker_resource_set_uri (resource, "nie:dataSource", identifier);
-
-		g_free (identifier);
-	}
-}
-
-static void
-miner_files_add_mount_info (TrackerMinerFiles *miner,
-                            TrackerResource   *resource,
-                            GFile             *file)
-{
-	TrackerMinerFilesPrivate *priv = TRACKER_MINER_FILES_GET_PRIVATE (miner);
-	TrackerStorageType storage_type;
-	const gchar *uuid;
-
-	uuid = tracker_storage_get_uuid_for_file (priv->storage, file);
-	if (!uuid)
-		return;
-
-	storage_type = tracker_storage_get_type_for_uuid (priv->storage, uuid);
-
-	tracker_resource_set_boolean (resource, "tracker:isRemovable",
-	                              (storage_type & TRACKER_STORAGE_REMOVABLE) != 0);
-	tracker_resource_set_boolean (resource, "tracker:isOptical",
-	                              (storage_type & TRACKER_STORAGE_OPTICAL) != 0);
-}
-
-static TrackerResource *
-miner_files_create_folder_information_element (TrackerMinerFiles *miner,
-                                               GFile             *file,
-                                               const gchar       *mime_type,
-                                               gboolean           create)
-{
-	TrackerResource *resource, *file_resource;
-	TrackerIndexingTree *indexing_tree;
-	gchar *urn, *uri;
-
-	/* Preserve URN for nfo:Folders */
-	urn = tracker_miner_fs_get_identifier (TRACKER_MINER_FS (miner),
-	                                       file, create, TRUE, NULL);
-	resource = tracker_resource_new (urn);
-	g_free (urn);
-
-	tracker_resource_set_string (resource, "nie:mimeType", mime_type);
-	tracker_resource_add_uri (resource, "rdf:type", "nie:InformationElement");
-
-	tracker_resource_add_uri (resource, "rdf:type", "nfo:Folder");
-	indexing_tree = tracker_miner_fs_get_indexing_tree (TRACKER_MINER_FS (miner));
-
-	if (tracker_indexing_tree_file_is_root (indexing_tree, file)) {
-		tracker_resource_add_uri (resource, "rdf:type", "tracker:IndexedFolder");
-		tracker_resource_set_boolean (resource, "tracker:available", TRUE);
-		tracker_resource_set_uri (resource, "nie:rootElementOf",
-		                          tracker_resource_get_identifier (resource));
-
-		miner_files_add_mount_info (miner, resource, file);
-	}
-
-	uri = g_file_get_uri (file);
-	file_resource = tracker_resource_new (uri);
-	tracker_resource_add_uri (file_resource, "rdf:type", "nfo:FileDataObject");
-	g_free (uri);
-
-	/* Laying the link between the IE and the DO */
-	tracker_resource_add_take_relation (resource, "nie:isStoredAs", file_resource);
-	tracker_resource_add_uri (file_resource, "nie:interpretedAs",
-				  tracker_resource_get_identifier (resource));
-
-	return resource;
-}
-
-static void
-miner_files_process_file (TrackerMinerFS      *fs,
-                          GFile               *file,
-                          GFileInfo           *file_info,
-                          TrackerSparqlBuffer *buffer,
-                          gboolean             create)
-{
-	TrackerMinerFilesPrivate *priv;
-	TrackerResource *resource = NULL, *folder_resource = NULL, *graph_file = NULL;
-	const gchar *mime_type, *graph;
-	gchar *parent_urn;
-	gchar *delete_properties_sparql = NULL;
-	time_t time_;
-	GFile *parent;
-	gchar *uri, *time_str;
-	gboolean is_directory;
-	GDateTime *modified;
-
-	priv = TRACKER_MINER_FILES (fs)->private;
+	TrackerMinerFilesPrivate *priv = TRACKER_MINER_FILES (fs)->private;
 
 	priv->start_extractor = TRUE;
-	uri = g_file_get_uri (file);
-	mime_type = g_file_info_get_content_type (file_info);
-
-	is_directory = (g_file_info_get_file_type (file_info) == G_FILE_TYPE_DIRECTORY ?
-	                TRUE : FALSE);
-	modified = g_file_info_get_modification_date_time (file_info);
-
-	if (!create && !is_directory) {
-		/* In case of update: delete all information elements for the given data object
-		 * and delete extractorHash, so we ensure the file is extracted again.
-		 */
-		delete_properties_sparql =
-			g_strdup_printf ("DELETE {"
-			                 "  GRAPH ?g {"
-			                 "    <%s> nie:interpretedAs ?ie . "
-			                 "    ?ie a rdfs:Resource . "
-			                 "  }"
-			                 "} WHERE {"
-			                 "  GRAPH ?g {"
-			                 "    <%s> nie:interpretedAs ?ie ."
-			                 "  }"
-			                 "}; "
-					 "DELETE WHERE {"
-					 "  GRAPH " DEFAULT_GRAPH " {"
-					 "    <%s> tracker:extractorHash ?h ."
-					 "  }"
-					 "}",
-			                 uri, uri, uri);
-	}
-
-	resource = tracker_resource_new (uri);
-
-	tracker_resource_add_uri (resource, "rdf:type", "nfo:FileDataObject");
-
-	parent = g_file_get_parent (file);
-	parent_urn = tracker_miner_fs_get_identifier (fs, parent, FALSE, TRUE, NULL);
-	g_object_unref (parent);
-
-	if (parent_urn) {
-		tracker_resource_set_uri (resource, "nfo:belongsToContainer", parent_urn);
-		g_free (parent_urn);
-	}
-
-	tracker_resource_set_string (resource, "nfo:fileName",
-	                             g_file_info_get_display_name (file_info));
-	tracker_resource_set_int64 (resource, "nfo:fileSize",
-	                            g_file_info_get_size (file_info));
-
-	time_str = g_date_time_format_iso8601 (modified);
-	tracker_resource_set_string (resource, "nfo:fileLastModified", time_str);
-	g_free (time_str);
-
-	time_ = (time_t) g_file_info_get_attribute_uint64 (file_info, G_FILE_ATTRIBUTE_TIME_ACCESS);
-	time_str = tracker_date_to_string (time_);
-	tracker_resource_set_string (resource, "nfo:fileLastAccessed", time_str);
-	g_free (time_str);
-
-	/* The URL of the DataObject (because IE = DO, this is correct) */
-	tracker_resource_set_string (resource, "nie:url", uri);
-
-	if (is_directory) {
-		folder_resource =
-			miner_files_create_folder_information_element (TRACKER_MINER_FILES (fs),
-								       file,
-								       mime_type,
-								       create);
-	}
-
-	miner_files_add_to_datasource (TRACKER_MINER_FILES (fs), file, resource, folder_resource);
-
-	graph = tracker_extract_module_manager_get_graph (mime_type);
-
-	if (graph) {
-		/* This mimetype will be extracted by some module, pre-fill the
-		 * nfo:FileDataObject in that graph.
-		 */
-		graph_file = tracker_resource_new (uri);
-		tracker_resource_add_uri (graph_file, "rdf:type", "nfo:FileDataObject");
-
-		time_str = g_date_time_format_iso8601 (modified);
-		tracker_resource_set_string (graph_file, "nfo:fileLastModified", time_str);
-		g_free (time_str);
-	}
-
-	if (delete_properties_sparql)
-		tracker_sparql_buffer_push_sparql (buffer, file, delete_properties_sparql);
-
-	tracker_sparql_buffer_push (buffer, file, DEFAULT_GRAPH, resource);
-
-	if (graph_file)
-		tracker_sparql_buffer_push (buffer, file, graph, graph_file);
-	if (folder_resource)
-		tracker_sparql_buffer_push (buffer, file, DEFAULT_GRAPH, folder_resource);
-
-	g_date_time_unref (modified);
-	g_object_unref (resource);
-	g_clear_object (&folder_resource);
-	g_clear_object (&graph_file);
-	g_free (uri);
+	tracker_miner_files_process_file (fs, file, info, buffer, create);
 }
 
 static void
-miner_files_process_file_attributes (TrackerMinerFS      *fs,
-                                     GFile               *file,
-                                     GFileInfo           *info,
-                                     TrackerSparqlBuffer *buffer)
+miner_files_process_file_attributes (TrackerMinerFS       *fs,
+                                     GFile                *file,
+                                     GFileInfo            *info,
+                                     TrackerSparqlBuffer  *buffer)
 {
-	TrackerResource *resource;
-	time_t time_;
-	gchar *uri, *time_str;
-	GDateTime *modified;
-
-	uri = g_file_get_uri (file);
-	resource = tracker_resource_new (uri);
-
-	if (!info) {
-		info = g_file_query_info (file,
-		                          G_FILE_ATTRIBUTE_TIME_MODIFIED ","
-		                          G_FILE_ATTRIBUTE_TIME_ACCESS,
-		                          G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
-		                          NULL, NULL);
-	}
-
-	/* Update nfo:fileLastModified */
-	modified = g_file_info_get_modification_date_time (info);
-	time_str = g_date_time_format_iso8601 (modified);
-	tracker_resource_set_string (resource, "nfo:fileLastModified", time_str);
-	g_date_time_unref (modified);
-	g_free (time_str);
-
-	/* Update nfo:fileLastAccessed */
-	time_ = (time_t) g_file_info_get_attribute_uint64 (info, G_FILE_ATTRIBUTE_TIME_ACCESS);
-	time_str = tracker_date_to_string (time_);
-	tracker_resource_set_string (resource, "nfo:fileLastAccessed", time_str);
-	g_free (time_str);
-
-	g_free (uri);
-
-	tracker_sparql_buffer_push (buffer, file, DEFAULT_GRAPH, resource);
-	g_object_unref (resource);
+	tracker_miner_files_process_file_attributes (fs, file, info, buffer);
 }
 
 static void
@@ -2229,94 +1960,12 @@ miner_files_finished (TrackerMinerFS *fs,
 	tracker_miner_files_check_unextracted (TRACKER_MINER_FILES (fs));
 }
 
-static void
-add_delete_sparql (GFile               *file,
-                   TrackerSparqlBuffer *buffer,
-                   gboolean             delete_self,
-                   gboolean             delete_children)
-{
-	GString *sparql;
-	gchar *uri;
-
-	g_return_if_fail (delete_self || delete_children);
-
-	uri = g_file_get_uri (file);
-
-	sparql = g_string_new (NULL);
-
-	if (delete_children) {
-		sparql = g_string_new ("DELETE { "
-				       "  GRAPH " DEFAULT_GRAPH " {"
-				       "    ?f a rdfs:Resource . "
-				       "  }"
-				       "  GRAPH ?g {"
-				       "    ?f a rdfs:Resource . "
-				       "    ?ie a rdfs:Resource . "
-				       "  }"
-				       "} WHERE {"
-				       "  GRAPH " DEFAULT_GRAPH " {"
-				       "    ?f a rdfs:Resource ; "
-				       "       nie:url ?u . "
-				       "  }"
-				       "  GRAPH ?g {"
-				       "    ?f a rdfs:Resource . "
-				       "    OPTIONAL { ?ie nie:isStoredAs ?f } . "
-				       "  }"
-				       "  FILTER (");
-
-		g_string_append_printf (sparql, "STRSTARTS (?u, \"%s/\")", uri);
-
-		g_string_append (sparql, ")}");
-	}
-
-	if (delete_self) {
-		const gchar *data_graphs[] = {
-			"tracker:Audio",
-			"tracker:Documents",
-			"tracker:Pictures",
-			"tracker:Software",
-			"tracker:Video",
-			"tracker:FileSystem",
-		};
-		gint i;
-
-		for (i = 0; i < G_N_ELEMENTS (data_graphs); i++) {
-			g_string_append_printf (sparql,
-						"DELETE { "
-						"  GRAPH %s {"
-						"    <%s> a rdfs:Resource . "
-						"    ?ie a rdfs:Resource . "
-						"  }"
-						"} WHERE {"
-						"  GRAPH " DEFAULT_GRAPH " {"
-						"    <%s> a rdfs:Resource . "
-						"    OPTIONAL { "
-						"      GRAPH %s {"
-						"        ?ie nie:isStoredAs <%s> "
-						"      }"
-						"    }"
-						"  }"
-						"} ",
-						data_graphs[i],
-						uri,
-						uri,
-						data_graphs[i],
-						uri);
-		}
-	}
-
-	g_free (uri);
-
-	tracker_sparql_buffer_push_sparql (buffer, file, sparql->str);
-	g_string_free (sparql, TRUE);
-}
-
 static void
 miner_files_remove_children (TrackerMinerFS      *fs,
                              GFile               *file,
                              TrackerSparqlBuffer *buffer)
 {
-	add_delete_sparql (file, buffer, FALSE, TRUE);
+	tracker_sparql_buffer_log_delete_content (buffer, file);
 }
 
 static void
@@ -2325,7 +1974,10 @@ miner_files_remove_file (TrackerMinerFS      *fs,
                          TrackerSparqlBuffer *buffer,
                          gboolean             is_dir)
 {
-	add_delete_sparql (file, buffer, TRUE, is_dir);
+	if (is_dir)
+		tracker_sparql_buffer_log_delete_content (buffer, file);
+
+	tracker_sparql_buffer_log_delete (buffer, file);
 }
 
 static void
@@ -2335,158 +1987,10 @@ miner_files_move_file (TrackerMinerFS      *fs,
                        TrackerSparqlBuffer *buffer,
                        gboolean             recursive)
 {
-	GString *sparql = g_string_new (NULL);
-	gchar *uri, *source_uri, *display_name, *container_clause = NULL;
-	gchar *path, *basename;
-	GFile *new_parent;
+	tracker_sparql_buffer_log_move (buffer, source_file, file);
 
-	uri = g_file_get_uri (file);
-	source_uri = g_file_get_uri (source_file);
-
-	path = g_file_get_path (file);
-	basename = g_filename_display_basename (path);
-	display_name = tracker_sparql_escape_string (basename);
-	g_free (basename);
-	g_free (path);
-
-	/* Get new parent information */
-	new_parent = g_file_get_parent (file);
-	if (new_parent) {
-		gchar *new_parent_id;
-		gboolean is_iri;
-
-		new_parent_id = tracker_miner_fs_get_identifier (fs, new_parent, FALSE, FALSE, &is_iri);
-
-		if (new_parent_id) {
-			container_clause =
-				g_strdup_printf ("; nfo:belongsToContainer %s%s%s",
-				                 is_iri ? "<" : "",
-				                 new_parent_id,
-				                 is_iri ? ">" : "");
-		}
-
-		g_free (new_parent_id);
-	}
-
-	/* Update nie:isStoredAs in the nie:InformationElement */
-	g_string_append_printf (sparql,
-	                        "DELETE { "
-	                        "  GRAPH ?g {"
-	                        "    ?ie nie:isStoredAs <%s> "
-	                        "  }"
-	                        "} INSERT {"
-	                        "  GRAPH ?g {"
-	                        "    ?ie nie:isStoredAs <%s> "
-	                        "  }"
-	                        "} WHERE {"
-	                        "  GRAPH ?g {"
-	                        "    ?ie nie:isStoredAs <%s> "
-	                        "  }"
-	                        "}; ",
-	                        source_uri, uri, source_uri);
-	/* Update tracker:FileSystem nfo:FileDataObject information */
-	g_string_append_printf (sparql,
-	                        "WITH " DEFAULT_GRAPH " "
-	                        "DELETE { "
-	                        "  <%s> a rdfs:Resource . "
-	                        "} INSERT { "
-	                        "  <%s> a nfo:FileDataObject ; "
-	                        "       nfo:fileName \"%s\" ; "
-	                        "       nie:url \"%s\" "
-	                        "       %s ; "
-	                        "       ?p ?o . "
-	                        "} WHERE { "
-	                        "  <%s> ?p ?o ; "
-	                        "  FILTER (?p != nfo:fileName && ?p != nie:url && ?p != nfo:belongsToContainer) . "
-	                        "} ",
-	                        source_uri,
-	                        uri, display_name, uri, container_clause,
-	                        source_uri);
-	/* Update nfo:FileDataObject in data graphs */
-	g_string_append_printf (sparql,
-	                        "DELETE { "
-	                        "  GRAPH ?g {"
-	                        "    <%s> a rdfs:Resource "
-	                        "  }"
-	                        "} INSERT {"
-	                        "  GRAPH ?g {"
-	                        "    <%s> a nfo:FileDataObject ; "
-	                        "         ?p ?o "
-	                        "  }"
-	                        "} WHERE {"
-	                        "  GRAPH ?g {"
-	                        "    <%s> ?p ?o "
-	                        "  }"
-	                        "}",
-	                        source_uri, uri, source_uri);
-	g_free (container_clause);
-
-	if (recursive) {
-		/* Update nie:isStoredAs in the nie:InformationElement */
-		g_string_append_printf (sparql,
-		                        "DELETE { "
-		                        "  GRAPH ?g {"
-		                        "    ?ie nie:isStoredAs ?f "
-		                        "  }"
-		                        "} INSERT {"
-		                        "  GRAPH ?g {"
-		                        "    ?ie nie:isStoredAs ?new_url "
-		                        "  }"
-		                        "} WHERE {"
-		                        "  GRAPH ?g {"
-		                        "    ?f a nfo:FileDataObject ."
-		                        "    ?ie nie:isStoredAs ?f ."
-		                        "    BIND (CONCAT (\"%s/\", SUBSTR (STR (?f), STRLEN (\"%s/\") + 1)) AS ?new_url) ."
-		                        "    FILTER (STRSTARTS (STR (?f), \"%s/\")) . "
-		                        "  }"
-		                        "}; ",
-		                        uri, source_uri, source_uri);
-		/* Update tracker:FileSystem nfo:FileDataObject information */
-		g_string_append_printf (sparql,
-		                        "WITH " DEFAULT_GRAPH " "
-		                        "DELETE { "
-		                        "  ?f a rdfs:Resource . "
-		                        "} INSERT { "
-		                        "  ?new_url a nfo:FileDataObject ; "
-		                        "       nie:url ?new_url ; "
-		                        "       ?p ?o . "
-		                        "} WHERE { "
-		                        "  ?f a nfo:FileDataObject ;"
-		                        "     ?p ?o . "
-		                        "  BIND (CONCAT (\"%s/\", SUBSTR (STR (?f), STRLEN (\"%s/\") + 1)) AS ?new_url) ."
-		                        "  FILTER (STRSTARTS (STR (?f), \"%s/\")) . "
-		                        "  FILTER (?p != nie:url) . "
-		                        "} ",
-		                        uri, source_uri, source_uri);
-		/* Update nfo:FileDataObject in data graphs */
-		g_string_append_printf (sparql,
-		                        "DELETE { "
-		                        "  GRAPH ?g {"
-		                        "    ?f a rdfs:Resource "
-		                        "  }"
-		                        "} INSERT {"
-		                        "  GRAPH ?g {"
-		                        "    ?new_url a nfo:FileDataObject ; "
-		                        "             ?p ?o ."
-		                        "  }"
-		                        "} WHERE {"
-		                        "  GRAPH ?g {"
-		                        "    ?f a nfo:FileDataObject ;"
-		                        "       ?p ?o ."
-		                        "    BIND (CONCAT (\"%s/\", SUBSTR (STR (?f), STRLEN (\"%s/\") + 1)) AS ?new_url) ."
-		                        "    FILTER (STRSTARTS (STR (?f), \"%s/\")) . "
-		                        "  }"
-		                        "}",
-		                        uri, source_uri, source_uri);
-	}
-
-	tracker_sparql_buffer_push_sparql (buffer, file, sparql->str);
-
-	g_free (uri);
-	g_free (source_uri);
-	g_free (display_name);
-	g_clear_object (&new_parent);
-	g_string_free (sparql, TRUE);
+	if (recursive)
+		tracker_sparql_buffer_log_move_content (buffer, source_file, file);
 }
 
 TrackerMiner *
@@ -2503,7 +2007,7 @@ tracker_miner_files_new (TrackerSparqlConnection  *connection,
 	                       "config", config,
 	                       "domain", domain,
 	                       "processing-pool-wait-limit", 1,
-	                       "processing-pool-ready-limit", 100,
+	                       "processing-pool-ready-limit", 800,
 	                       "file-attributes", FILE_ATTRIBUTES,
 	                       NULL);
 }
@@ -2513,20 +2017,21 @@ remove_files_in_removable_media_cb (GObject      *object,
                                     GAsyncResult *result,
                                     gpointer      user_data)
 {
-	GError *error = NULL;
+	g_autofree GError *error = NULL;
 
-	tracker_sparql_connection_update_finish (TRACKER_SPARQL_CONNECTION (object), result, &error);
+	tracker_sparql_statement_update_finish (TRACKER_SPARQL_STATEMENT (object),
+	                                        result, &error);
 
-	if (error) {
+	if (error)
 		g_critical ("Could not remove files in volumes: %s", error->message);
-		g_error_free (error);
-	}
 }
 
 static gboolean
 miner_files_in_removable_media_remove_by_type (TrackerMinerFiles  *miner,
                                                TrackerStorageType  type)
 {
+	TrackerSparqlConnection *conn;
+	g_autoptr (TrackerSparqlStatement) stmt = NULL;
 	gboolean removable;
 	gboolean optical;
 
@@ -2534,92 +2039,42 @@ miner_files_in_removable_media_remove_by_type (TrackerMinerFiles  *miner,
 	optical = TRACKER_STORAGE_TYPE_IS_OPTICAL (type);
 
 	/* Only remove if any of the flags was TRUE */
-	if (removable || optical) {
-		GString *queries;
+	if (!removable && !optical)
+		return FALSE;
 
-		g_debug ("  Removing all resources in store from %s ",
-		         optical ? "optical discs" : "removable devices");
+	g_debug ("  Removing all resources in store from %s ",
+	         optical ? "optical discs" : "removable devices");
 
-		queries = g_string_new ("");
+	conn = tracker_miner_get_connection (TRACKER_MINER (miner));
+	stmt = tracker_load_statement (conn, "delete-mountpoints-by-type.rq", NULL);
 
-		/* Delete all resources where nie:dataSource is a volume
-		 * of the given type */
-		g_string_append_printf (queries,
-		                        "DELETE { "
-		                        "  ?f a rdfs:Resource . "
-		                        "  GRAPH ?g {"
-		                        "    ?ie a rdfs:Resource "
-		                        "  }"
-		                        "} WHERE { "
-		                        "  ?v a tracker:IndexedFolder ; "
-		                        "     tracker:isRemovable %s ; "
-		                        "     tracker:isOptical %s . "
-		                        "  ?f nie:dataSource ?v . "
-		                        "  GRAPH ?g {"
-		                        "    ?ie nie:isStoredAs ?f "
-		                        "  }"
-		                        "}",
-		                        removable ? "true" : "false",
-		                        optical ? "true" : "false");
+	tracker_sparql_statement_bind_boolean (stmt, "isRemovable", removable);
+	tracker_sparql_statement_bind_boolean (stmt, "isOptical", optical);
+	tracker_sparql_statement_update_async (stmt, NULL,
+	                                       remove_files_in_removable_media_cb,
+	                                       NULL);
 
-		tracker_sparql_connection_update_async (tracker_miner_get_connection (TRACKER_MINER (miner)),
-		                                        queries->str,
-		                                        NULL,
-		                                        remove_files_in_removable_media_cb,
-		                                        NULL);
-
-		g_string_free (queries, TRUE);
-
-		return TRUE;
-	}
-
-	return FALSE;
+	return TRUE;
 }
 
 static void
 miner_files_in_removable_media_remove_by_date (TrackerMinerFiles  *miner,
                                                const gchar        *date)
 {
-	GString *queries;
+	TrackerSparqlConnection *conn;
+	g_autoptr (TrackerSparqlStatement) stmt = NULL;
 
 	g_debug ("  Removing all resources in store from removable or "
 	         "optical devices not mounted after '%s'",
 	         date);
 
-	queries = g_string_new ("");
+	conn = tracker_miner_get_connection (TRACKER_MINER (miner));
+	stmt = tracker_load_statement (conn, "delete-mountpoints-by-date.rq", NULL);
 
-	/* Delete all resources where nie:dataSource is a volume
-	 * which was last unmounted before the given date */
-	g_string_append_printf (queries,
-	                        "DELETE { "
-				"  GRAPH " DEFAULT_GRAPH " {"
-	                        "    ?f a rdfs:Resource . "
-				"  }"
-	                        "  GRAPH ?g {"
-	                        "    ?ie a rdfs:Resource "
-	                        "  }"
-	                        "} WHERE { "
-				"  GRAPH " DEFAULT_GRAPH " {"
-	                        "    ?v a tracker:IndexedFolder ; "
-	                        "       tracker:isRemovable true ; "
-	                        "       tracker:available false ; "
-	                        "       tracker:unmountDate ?d . "
-	                        "    ?f nie:dataSource ?v . "
-	                        "    FILTER ( ?d < \"%s\"^^xsd:dateTime) "
-				"  }"
-	                        "  GRAPH ?g {"
-	                        "    ?ie nie:isStoredAs ?f "
-	                        "  }"
-	                        "}",
-	                        date);
-
-	tracker_sparql_connection_update_async (tracker_miner_get_connection (TRACKER_MINER (miner)),
-	                                        queries->str,
-	                                        NULL,
-	                                        remove_files_in_removable_media_cb,
-	                                        NULL);
-
-	g_string_free (queries, TRUE);
+	tracker_sparql_statement_bind_string (stmt, "unmountDate", date);
+	tracker_sparql_statement_update_async (stmt, NULL,
+	                                       remove_files_in_removable_media_cb,
+	                                       NULL);
 }
 
 static void
@@ -2928,3 +2383,9 @@ tracker_miner_files_set_mtime_checking (TrackerMinerFiles *mf,
 {
 	mf->private->mtime_check = mtime_check;
 }
+
+TrackerStorage *
+tracker_miner_files_get_storage (TrackerMinerFiles *mf)
+{
+	return mf->private->storage;
+}
